function [im1_idx, im2_idx] = flow2idx(im1, im2, forward_flow, forward_error, backward_flow, backward_error)
    % use computed flow to get correspondent pixel index
    % use two flag variables make it easier to see both result when debugging.
if forward_error < backward_error
    flag_forward = true; flag_backward = false;
else
    flag_forward = false; flag_backward = true;
end
% flag_forward = false; flag_backward = true;

assert(isequal([size(im1,1), size(im1, 2)], [size(forward_flow, 1), size(forward_flow, 2)]));
assert(isequal([size(im2,1), size(im2, 2)], [size(backward_flow, 1), size(backward_flow, 2)]));

%% get correspondent pixels across images
assert(flag_forward ~= flag_backward);

[height2,width2,~]=size(im2);
[height1,width1,~]=size(im1);

if flag_forward
    vx = forward_flow(:,:,1); vy = forward_flow(:,:,2);
    [XX,YY]=meshgrid(1:width1,1:height1);

    % [xx,yy]=meshgrid(1:width2,1:height2);
    % XX=XX+vx; YY=YY+vy;
    XX1=XX+vx; YY1=YY+vy;

    XX1(XX1<1)=NaN; YY1(YY1<1)=NaN;
    XX1(XX1>width2)=NaN; YY1(YY1>height2)=NaN;
    % XX is pixel index of im1
    % XX1 is correspondent pixel index of im2
end

if flag_backward
    vx = backward_flow(:,:,1); vy = backward_flow(:,:,2);
    [XX1,YY1]=meshgrid(1:width2,1:height2);
    % XX1 is pixel index of im2
    % XX is corrrespondent pixel index of im1

    XX = XX1 + vx;
    YY = YY1 + vy;
    XX(XX<1)=NaN; YY(YY<1)=NaN;
    XX(XX>width1)=NaN; YY(YY>height1)=NaN;
    % remove invalid pixel index
end

im1_idx = sub2ind([height1, width1], YY, XX);
im2_idx = sub2ind([height2, width2], YY1, XX1);
im1_idx = im1_idx(:); im2_idx = im2_idx(:);
% Remove some out of border points
invalid = isnan(im2_idx) | isnan(im1_idx);
im1_idx(invalid) = []; im2_idx(invalid) = [];
end