function [epeMean, epeStd, angleMean, angleStd] = measure_error(gtFlow, cFlow)
    % MEASURE_ERROR compute EPE and AE
    % compute Endpoint error (mean, std)
    % Angular error (mean, std)
    margin = 1; % crop one pixel margin from evaluation, because unreliable sift feature in the border.
    [epeMean, epeStd] = compute_edp_error(gtFlow, cFlow, margin); 
    [angleMean, angleStd] = compute_angle_error(gtFlow, cFlow, margin);
end

% Reference: Baker, Simon, et al. "A database and evaluation methodology for optical flow." 
% International Journal of Computer Vision 92.1 (2011): 1-31.
function [angleMean, angleStd] = compute_angle_error(gtFlow, cFlow, margin)
    gtFlow = gtFlow(1+margin:end-margin, 1+margin:end-margin, :);
    cFlow = cFlow(1+margin:end-margin, 1+margin:end-margin, :);
    [h, w, c] = size(gtFlow);
    z = ones(h, w);
    gtFlow(:,:,3) = z;
    cFlow(:,:,3) = z;
    
    % normalize flow
    gtFlow = gtFlow ./ repmat(sqrt(sum(gtFlow.^2, 3)), [1, 1, 3]);
    cFlow = cFlow ./ repmat(sqrt(sum(cFlow.^2, 3)), [1, 1, 3]);
    
    diff = acos(sum(gtFlow .* cFlow, 3));
    diff = diff(:);
    diff = diff .* 180 / pi;
    
    
    angleMean = mean(diff);
    angleStd = std(diff);
end

function [epeMean, epeStd] = compute_edp_error(gtFlow, cFlow, margin)
    % crop margin pixel;
    gtFlow = gtFlow(1+margin:end-margin, 1+margin:end-margin, :);
    cFlow = cFlow(1+margin:end-margin, 1+margin:end-margin, :);
    
    gtU = gtFlow(:,:,1); gtV = gtFlow(:,:,2);
%     UNKNOWN_FLOW_THRESH = 1e8;
%     idxUnknown = (abs(gt_u)> UNKNOWN_FLOW_THRESH) | (abs(gt_v)> UNKNOWN_FLOW_THRESH) ;
%     gt_u(idxUnknown) = 0;
%     gt_v(idxUnknown) = 0;

    u = cFlow(:,:,1); v = cFlow(:,:,2);
    
    diff = sqrt((gtU - u).^2 + (gtV - v).^2); diff = diff(:);
    epeMean = mean(diff);
    epeStd = std(diff);
end
